@@ -428,13 +428,14 @@ read_ahead:
 					space, i);
 				break;
 			case DB_DECRYPTION_FAILED:
+			case DB_PAGE_CORRUPTED:
 				ib_logf(IB_LOG_LEVEL_ERROR,
-					"Random readahead failed to decrypt page "
+					"Random readahead failed to decrypt page or page corrupted "
 					ULINTPF ":" ULINTPF ".",
 					i, space);
 				break;
 			default:
-				ut_error;
+				ib_logf(IB_LOG_LEVEL_FATAL, "Error %u (%s) in random readahead", err, ut_strerr(err));
 			}
 		}
 	}
@@ -570,13 +571,14 @@ buf_read_page_async(
 		break;
 
 	case DB_DECRYPTION_FAILED:
+	case DB_PAGE_CORRUPTED:
 		ib_logf(IB_LOG_LEVEL_ERROR,
-			"Async page read failed to decrypt page "
+			"Async page read failed to decrypt page or page corrupted "
 			ULINTPF ":" ULINTPF ".",
 			space, offset);
 		break;
 	default:
-		ut_error;
+		ib_logf(IB_LOG_LEVEL_FATAL, "Error %u (%s) in async page read", err, ut_strerr(err));
 	}
 
 	srv_stats.buf_pool_reads.add(count);
@@ -863,13 +865,14 @@ buf_read_ahead_linear(
 				break;
 
 			case DB_DECRYPTION_FAILED:
+			case DB_PAGE_CORRUPTED:
 				ib_logf(IB_LOG_LEVEL_ERROR,
-					"Linear readahead failed to decrypt page "
+					"Linear readahead failed to decrypt page or page corrupted"
 					ULINTPF ":" ULINTPF ".",
 					i, space);
 				break;
 			default:
-				ut_error;
+				ib_logf(IB_LOG_LEVEL_FATAL, "Error %u (%s) in linear readahead", err, ut_strerr(err));
 			}
 		}
 	}
@@ -963,13 +966,14 @@ tablespace_deleted:
 			ibuf_delete_for_discarded_space(space_ids[i]);
 			break;
 		case DB_DECRYPTION_FAILED:
+		case DB_PAGE_CORRUPTED:
 			ib_logf(IB_LOG_LEVEL_ERROR,
-				"Failed to decrypt insert buffer page "
+				"Failed to decrypt insert buffer page or page corrupted "
 				ULINTPF ":" ULINTPF ".",
 				space_ids[i], page_nos[i]);
 			break;
 		default:
-			ut_error;
+			ib_logf(IB_LOG_LEVEL_FATAL, "Error %u (%s) in insert buffer read", err, ut_strerr(err));
 		}
 	}
 
